@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 2005, 2006, 2007, 2008 by                                     */
+/*  Copyright 2005, 2006, 2007, 2008, 2009 by                               */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -19,6 +19,7 @@
 
 #include FT_BITMAP_H
 
+#include "common.h"
 #include "ftcommon.h"
 
 #include <stdio.h>
@@ -1018,58 +1019,27 @@
 
 
   void
-  FTDemo_String_Set( FTDemo_Handle*        handle,
-                     const unsigned char*  string )
+  FTDemo_String_Set( FTDemo_Handle*  handle,
+                     const char*     string )
   {
-    const unsigned char*  p = string;
-    unsigned long         codepoint;
-    unsigned char         in_code;
-    int                   expect;
-    PGlyph                glyph = handle->string;
+    const char*     p = string;
+    const char*     end = p + strlen( string );
+    unsigned long   codepoint;
+    int             ch;
+    int             expect;
+    PGlyph          glyph = handle->string;
 
 
     handle->string_length = 0;
     codepoint = expect = 0;
 
-    while ( *p )
+    for (;;)
     {
-      in_code = *p++ ;
+      ch = utf8_next( &p, end );
+      if ( ch < 0 )
+        break;
 
-      if ( in_code >= 0xC0 )
-      {
-        if ( in_code < 0xE0 )           /*  U+0080 - U+07FF   */
-        {
-          expect = 1;
-          codepoint = in_code & 0x1F;
-        }
-        else if ( in_code < 0xF0 )      /*  U+0800 - U+FFFF   */
-        {
-          expect = 2;
-          codepoint = in_code & 0x0F;
-        }
-        else if ( in_code < 0xF8 )      /* U+10000 - U+10FFFF */
-        {
-          expect = 3;
-          codepoint = in_code & 0x07;
-        }
-        continue;
-      }
-      else if ( in_code >= 0x80 )
-      {
-        --expect;
-
-        if ( expect >= 0 )
-        {
-          codepoint <<= 6;
-          codepoint  += in_code & 0x3F;
-        }
-        if ( expect >  0 )
-          continue;
-
-        expect = 0;
-      }
-      else                              /* ASCII, U+0000 - U+007F */
-        codepoint = in_code;
+      codepoint = ch;
 
       if ( handle->encoding != FT_ENCODING_NONE )
         glyph->glyph_index = FTDemo_Get_Index( handle, codepoint );
